@@ -1,14 +1,31 @@
 from typing import  List
 import pandas as pd
+from xaif_templates import XAIF
+
+#python3 -m build  
+# twine upload dist/*   
 
     
 class AIF:
     def __init__(self, xaif):
+        if isinstance(xaif,str):
+            self.xaif = self.initilise_empty(xaif)
         self.xaif = xaif
         self.aif = xaif.get('AIF')
         self.nodes = self.aif.get('nodes')
         self.locutions = self.aif.get('locutions')
         self.participants = self.aif.get('participants')
+    def initilise_empty(self,xaif):
+            node_id, person_id = 0, 0
+            aif, json_aif, OVA = {}, {}, {}      
+            text_with_span = ""
+            nodes, edges, schemefulfillments, descriptorfulfillments, participants, locutions = [], [], [], [], [], []
+            speakers_and_turns = xaif                  
+            nodes, locutions, participants, text_with_span, node_id, person_id = self.create_turn_entry(
+                nodes, node_id, person_id, text_with_span, speakers_and_turns, locutions, participants, False)
+            return XAIF.format(nodes, edges, locutions, schemefulfillments, descriptorfulfillments, participants, OVA, text_with_span, aif, extended_json_aif)
+
+
     def is_valid_json_aif(self,):
         if 'nodes' in self.aif  and 'locutions' in self.aif  and 'edges' in self.aif :
             return True
@@ -21,11 +38,9 @@ class AIF:
             if nodes_entry['type'] == "L":
                 return True
         return False
-    
-
     def get_next_max_id(self, component_type, id_key_word):
         """
-       Takes a list of nodes (edges) and returns the maximum node/edge ID.
+       Takes a list of nodesor/edges and returns the maximum node/edge ID.
         Arguments:
         - nodes/edges (List[Dict]): a list of nodes/edges, where each node is a dictionary containing a node/edge ID
         Returns:
@@ -33,6 +48,8 @@ class AIF:
         """
         component_entries = self.aif.get(component_type,[])
         max_id, lef_n_id, right_n_id = 0, 0, ""
+        if len(component_entries) == 0:
+            return 0
         if isinstance(component_entries[0][id_key_word],str): # check if the node id is a text or integer
             if "_" in component_entries[0][id_key_word]:
                 for node in component_entries:
@@ -55,13 +72,15 @@ class AIF:
                 temp_id = node[id_key_word]     
                 if temp_id > max_id:
                     max_id = temp_id   
-            return max_id+1
+            return max_id+1   
+
+
         
 
 
     def get_speaker(self, node_id: int) -> str:
         """
-        Takes a node ID, a list of locutions, and a list of participants, and returns the name of the participant who spoke the locution with the given node ID, or "None" 
+        Takes a node ID and returns the name of the participant who spoke the locution with the given node ID, or "None" 
         if the node ID is not found.
 
         Arguments:
@@ -105,12 +124,14 @@ class AIF:
             self._add_argument_relation(*args)
         elif component_type == 'segment':
             self._add_segment(*args)
+        elif component_type == 'locution':
+            self._add_Lnode(*args)
         else:
             raise ValueError("Invalid component type. Supported types are 'argument_relation' and 'segment'.")
 
     
-    def _add_argument_relation(self, prediction, index1, index2):
-
+    def _add_argument_relation(self, prediction, index1, index2,AR_text=None):
+        AR_type = prediction
         if prediction == "RA":
             AR_text = "Default Inference"
             AR_type = "RA"
@@ -127,9 +148,27 @@ class AIF:
         edge_id = self.get_next_max_id('edges', 'edgeID')
         self.aif['edges'].append({'fromID': node_id, 'toID': index2,'edgeID':edge_id})
 
+    def _add_Lnode(self, text, speaker):       
+        
+        first_name_last_name = speaker.split()
+        first_n, last_n = first_name_last_name[0], first_name_last_name[1]
 
 
-    def _add_segment(self, Lnode_ID, segment):       
+        node_id = self.get_next_max_id('nodes', 'nodeID')
+        self.xaif['AIF']['nodes'].append({'text': text, 'type':'L','nodeID': node_id})	
+        self.xaif['AIF']['locutions'].append({'personID': 1, 'nodeID': node_id})
+        # Loop through each participant and check if their participant ID matches the person ID from the locution
+        if not any(participant['firstname'] == first_n and participant['surname'] == last_n for participant in self.xaif['AIF']['participants']):
+            self.xaif['AIF']['participants'].append(
+                    {
+                    "participantID": 1,                                
+                    "firstname": "Default",                                
+                    "surname": "Speaker"
+                    }
+            )
+
+
+    def _add_segment(self, Lnode_ID, segments):       
         speaker, speaker_id = "", None		
         if self.xaif['AIF']['participants']:
             speaker, speaker_id = self.get_speaker(Lnode_ID)
@@ -141,10 +180,13 @@ class AIF:
                 speaker = first_n+" " + last_n
         else:
             first_n, last_n  = "None", "None"
+        for segment in segments:
+            node_id = self.get_next_max_id('nodes', 'nodeID')
+            self.xaif['AIF']['nodes'].append({'text': segment, 'type':'L','nodeID': node_id})		
+            self.xaif['AIF']['locutions'].append({'personID': speaker_id, 'nodeID': node_id})
+
+
 
-        node_id = self.get_next_max_id('nodes', 'nodeID')
-        self.xaif['AIF']['nodes'].append({'text': segment, 'type':'L','nodeID': node_id})		
-        self.xaif['AIF']['locutions'].append({'personID': speaker_id, 'nodeID': node_id})
         self.remove_entry(Lnode_ID)
     
     def get_i_node_ya_nodes_for_l_node(self, Lnode_ID):
@@ -306,5 +348,62 @@ class AIF:
                     df = pd.DataFrame(pairs, columns=['proposition1_id', 'proposition1_text', 'proposition2_id', 'proposition2_text'])
                     df['relation'] = 'None'  # Set default value for 'relation'
             return df
+    def create_turn_entry(
+        self,
+        nodes, 
+        node_id,
+        person_id, 
+        text_with_span,
+        propositions,
+        locutions,
+        participants,
+        dialogue 
+        ):
+        if dialogue:
+            for first_and_last_names, proposition in propositions:
+                first_last_names = first_and_last_names.split()
+                first_names, last_names = "None", "None"
+                if len(first_last_names) > 1:
+                    first_names,last_names = first_last_names[0],first_last_names[1]
+                else:
+                    first_names, last_names = first_last_names[0],"None"
+                text = proposition.replace("\n","")
+                nodes.append({'text': text, 'type':'L','nodeID': node_id})
+                locutions.append({'personID': person_id, 'nodeID': node_id})
+                # Check if the entry already exists based on first name and surname
+                if not any(participant['firstname'] == first_names and participant['surname'] == last_names for participant in participants):
+                    participants.append({
+                        "participantID": person_id,
+                        "firstname": first_names,
+                        "surname": last_names
+                    })
+                text_with_span = text_with_span+" "+first_names+" "+last_names+" "+"<span class=\"highlighted\" id=\""+str(node_id)+"\">"+text+"</span>.<br><br>"
+                node_id = node_id + 1 
+                person_id = person_id + 1
+
+
+        else:
+            text = propositions.replace("\n","")
+            speaker = "Default Speaker"
+            nodes.append({'text': text, 'type':'L','nodeID': node_id})	
+            locutions.append({'personID': 1, 'nodeID': node_id})
+            if not any(participant['firstname'] == "Default" and participant['surname'] == "Speaker" for participant in participants):
+                participants.append(
+                        {
+                        "participantID": 1,                                
+                        "firstname": "Default",                                
+                        "surname": "Speaker"
+                        }
+                    )	
+            text_with_span=text_with_span+" "+speaker+" "+"<span class=\"highlighted\" id=\""+str(node_id)+"\">"+text+"</span>.<br><br>"
+            node_id = node_id + 1
+        return (
+            nodes, 
+            locutions,
+            participants, 
+            text_with_span, 
+            node_id,
+            person_id
+            )
 
 
